//*******************************************************************************************************
//Instructables Shop Bot Contest Entry - StorageBot, Robotic Voice Controlled Storage Bins
//Danh Trinh, Feb 2012
//Please see library files for appropriate credits given to the authors.
//*******************************************************************************************************

#include "TimerOne.h"    //timer one interrupt library
#include "LPD8806.h"     //led strip library
#include "SPI.h"         //SPI library to control led strip

volatile unsigned int servo_timer, servo_high_duration;              //variables for generating servo pulse
volatile unsigned long steps_x, steps_z, steps_x_set, steps_z_set;   //variables for controlling stepper positions
volatile byte speed_z, speed_x, speed_zSet, speed_xSet;              //variables for controlling stepper velocities
volatile unsigned long steps_x_absolute, steps_z_absolute;            //store absolute position of stepper motors in steps

boolean scan_flag;                                           //true if running LED scan sweep in main loop

volatile boolean x_dir, z_dir;                                       //stores stepper motor direction

volatile boolean x_motion_done, z_motion_done;                       //flags store when stepper motion is done

volatile boolean interrupt_flag = false;                             //controls high duration of stepper direction pulses 
    
LPD8806 strip = LPD8806(74);                                         //initialize number of LEDS in strip
//LPD8806 strip = LPD8806(30);                                         //initialize number of LEDS in strip

union serial_unsigned_long                                           //stores unsigned long serial data
{
  unsigned long int_4_bytes;
  byte read_byte[4];
}
unsigned_long_data;

//************************************************************************************************************
//Initializations
//************************************************************************************************************
void setup()
{
  servo_timer = 0;                 //variable for storing 20ms refresh time of servo pulse = 800 interrupts @ 25us per interrupt
  servo_high_duration = 93;        //variable for storing high pulse duration of servo pulse = 60 interrupts = 1.5 ms high pulse, servo centered
                                   //93 interrupts = rack retracted
                                
  steps_x = 0;                     //initialize stepper position variables 
  steps_z = 0;
  steps_x_absolute = 10000;
  steps_z_absolute = 10000;
  
  speed_x = 10;                    //initialize stepper velocity variables
  speed_xSet = speed_x;
  speed_z = 10;
  speed_zSet = speed_z;
  
  x_dir = false;                      //initialize x stepper motor direction, 0 = left, 1 = right
  z_dir = false;                      //initialize z stepper motor direction, 0 = up, 1 = down
  x_motion_done = false;
  z_motion_done = false;

  scan_flag = false;
  
  Serial.begin(38400);             //set serial baud rate

  digitalWrite(4, HIGH);           //x stepper DIR initialization
  digitalWrite(5, LOW);            //x stepper PULSE initialization 
  digitalWrite(6, HIGH);           //z stepper DIR initialization
  digitalWrite(7, LOW);            //z stepper PULSE initialization
  digitalWrite(8, LOW);            //SERVO pin initialization
  
  pinMode(2, INPUT);               //z axis limit switch
  pinMode(3, INPUT);               //x axis limit switch 
  pinMode(4, OUTPUT);              //x stepper direction pin
  pinMode(5, OUTPUT);              //x stepper pulse pin
  pinMode(6, OUTPUT);              //z stepper direction pin  
  pinMode(7, OUTPUT);              //z stepper pulse pin
  pinMode(8, OUTPUT);              //SERVO pulse pin
  
  // Start up the LED strip
  strip.begin();
//  for (int i=0; i < strip.numPixels(); i++) {
//    strip.setPixelColor(i, 0);           // turn all pixels off
//  }
  
  led_blue(5);                            //make all leds on with light blue
  
  strip.show();                          // Update the strip, to start they are all 'off'
  
  Timer1.initialize(25);                 // initialize timer1, value in us
  Timer1.attachInterrupt(ISR_steppers);  // attaches timer overflow interrupt

}

//*****************************************************************************************************************************************
//This interrupt sub routine runs every 25us. It generates the pulses for the servo and the stepper motor pulse signal.
//*****************************************************************************************************************************************
void ISR_steppers()
{
  //generat SERVO motor pulse
  servo_timer++;

  if (servo_timer == servo_high_duration) {digitalWrite(8, LOW);}
  if (servo_timer == 800) {digitalWrite(8, HIGH); servo_timer = 0;}
  
  //generate stepper STEP pulse
  if (interrupt_flag)                                               //this flag logic makes high STEP pulse duration equal to only 1 interrupt pulse
  {
    if (steps_z)
    {
      z_motion_done = false;                                        //z motion in progress
      if (speed_z == 1) 
      {                                                              //set STEP pin high when countdown equals 1 & reload countdown value
        digitalWrite(5, HIGH); 
        speed_z = speed_zSet; 
        steps_z--;                                                   //update number of relative steps
        if (z_dir)                                                    //update absolute steps 
          {steps_z_absolute++;}
        else
          {steps_z_absolute--;}
      } 
      else 
      {speed_z--;}                                                  //count down   
    }    
    else 
    {z_motion_done = true;}                                        //z motion complete
    
    if (steps_x)
    { 
      x_motion_done = false;                                        //x motion active
      if (speed_x == 1)                                             //set STEP pin high when countdown equals 1 & reload countdown value
      {
        digitalWrite(7, HIGH); 
        speed_x = speed_xSet; 
        steps_x--;                                                  //update number of relative steps
        if (x_dir)                                                    //update absolute steps 
          {steps_x_absolute++;}
        else
          {steps_x_absolute--;}
      }      
      else 
      {speed_x--;}                                                  //count down
    }                     
    else
    {x_motion_done = true;}                                         //x motion complete
    
    interrupt_flag = false;              
  }
  else
  {     
   digitalWrite(5, LOW);                                           //STEP pins are set to low most of the time               
   digitalWrite(7, LOW);
    
   interrupt_flag = true;
  }
}  

//**********************************************************************************************************************************
//MAIN LOOP
//**********************************************************************************************************************************
void loop()
{  
if (Serial.available() > 0)                        //check for serial data
{
  switch (Serial.read())                           //interpret first byte
  {
    //stop both stepper motors 
    case 's':                                                    
       cli();                                      //atomic operation
       steps_z = 0;                                //clear steps variable
       steps_x = 0;                                //clear steps variable
       sei();     
     break;
     
     //set servo position, range is typically 40 to 80
     case 'a':
       while(Serial.available() < 1);              //get one byte position value
       servo_high_duration = Serial.read();        //update servo position 
     break;
     
     //move z stepper down
     case 'd':
       z_dir = true;                       //indicates z down direction
       digitalWrite(4, HIGH);             //set z stepper direction pin high
       get_serial_unsigned_long();        //get serial 4 byte position value
       cli();                             //atomic operation
       steps_z = unsigned_long_data.int_4_bytes;  //update steps_z variable with new 4 byte value 
       sei();     
     break;
     
     //move z stepper up
     case 'u':
       z_dir = false;                      //indicates z up direction
       digitalWrite(4, LOW);              //set z stepper direction pin low
       get_serial_unsigned_long();        //update steps_z variable with new 4 byte value 
       cli();                             //atomic operation
       steps_z = unsigned_long_data.int_4_bytes;  //update steps_z variable with new 4 byte value 
       sei();     
     break;
     
     //move x stepper right
     case 'r':
       x_dir = true;                         //indicates x right direction
       digitalWrite(6, LOW);              //set x stepper direction pin low
       get_serial_unsigned_long();        //update steps_z variable with new 4 byte value 
       cli();                             //atomic operation
       steps_x = unsigned_long_data.int_4_bytes;  //update steps_z variable with new 4 byte value 
       sei();     
     break;
     
     //move x stepper left
     case 'l':
       x_dir = false;                          //indicates x left direction
       digitalWrite(6, HIGH);              //set x stepper direction pin high
       get_serial_unsigned_long();         //update steps_z variable with new 4 byte value 
       cli();                              //atomic operation
       steps_x = unsigned_long_data.int_4_bytes;   //update steps_z variable with new 4 byte value 
       sei();      
     break;   
     
     //set z stepper speed, smaller integer is faster speed
     case 'q':
       while(Serial.available() < 1);
       speed_zSet = Serial.read();
     break;
     
     //set x stepper speed, smaller integer is faster speed
     case 'p':
       while(Serial.available() < 1);
       speed_xSet = Serial.read();
     break;
     
     //horizontal LED on
     case 'h': 
       while(Serial.available() < 1);
       set_column_led(strip.Color(0,0,127), Serial.read());
     break;
     
     //horizontal LED off
     case 'i': 
       while(Serial.available() < 1);
       set_column_led(0, Serial.read());
     break;
     
     //vertical LED off
     case 'v': 
       while(Serial.available() < 1);
       set_row_led(strip.Color(0,0,127), Serial.read());   
     break;
     
     //vertical LED off
     case 'w': 
       while(Serial.available() < 1);
       set_row_led(0, Serial.read());    
     break;
  
     //move z axis to home position
     case 'j':
       home_z_axis();
     break;
     
     //move x axis to home position
     case 'k':
       home_x_axis();
     break;
     
     //read z limit switch
     case 'm':
       read_z_switch();
     break;
     
     //read x limit switch
     case 'n':
       read_x_switch();
     break;
     
     //return x absolute position
     case 'b':
       Serial.println(steps_x_absolute);
     break;
     
     //return z absolute position
     case 'c':
       Serial.println(steps_z_absolute);
     break;
     
     //return "1" when stepper motions completed
     case 'e':
       while(!x_motion_done || !z_motion_done) {if (scan_flag) {scanner(0,0,16, 10);};}       //wait for stepper motions to complete
       Serial.print("1");
     break;
     
     //perform led sweep along top row
     case 'z':
       scanner(0,0,16, 10);        // blue, fast
     break;
     
     //turn all leds on to light blue
     case 'y':
       led_blue(5);
     break;
    
     //turn all leds off
     case 'x':
       led_blue(0);
     break;
    
    //mainloop led scan on
     case 'f':
       scan_flag = true;
     break;
 
     //main loop led scan off
     case 'g':
       scan_flag = false;
     break;    
     }
  }

  if (scan_flag) {scanner(0,0,16, 10);}        //run led scan 

}

//*****************************************************************************************************
//get 4 bytes of serial data and store as unsigned long
void get_serial_unsigned_long()
{
   while(Serial.available() < 4);
   unsigned_long_data.read_byte[0] = Serial.read();
   unsigned_long_data.read_byte[1] = Serial.read();
   unsigned_long_data.read_byte[2] = Serial.read();
   unsigned_long_data.read_byte[3] = Serial.read(); 
}

//****************************************************************************************************
//LED test mode
//sequences through all the leds in the strip using the color blue
void led_blue(uint8_t y)
{
  for (int x = 0; x < 75; x++)
  {
  strip.setPixelColor(x, strip.Color(0,0,y));
  }
  strip.show();
}

//******************************************************************************************************************
//for a 3x3 storage bin system, the total# of rows = 27 (or 9 x 3) since each storage bin has 9 vertical drawers
void set_row_led(uint32_t c, uint8_t x)
{
  
  for (int i=30; i < 75; i++) 
  {
    strip.setPixelColor(i, 0);           // turn all pixels off
  }
  
  switch (x)                           //interpret first byte
  {
    //stop both stepper motors 
    case 1:                                                    
      strip.setPixelColor(30, c);
      strip.setPixelColor(31, c);
    break;

    case 2:                                                    
      strip.setPixelColor(31, c);
      strip.setPixelColor(32, c);
      strip.setPixelColor(33, c);
    break;
  
    case 3:                                                    
      strip.setPixelColor(33, c);
      strip.setPixelColor(34, c);
    break;
 
    case 4:                                                    
      strip.setPixelColor(34, c);
      strip.setPixelColor(35, c);
      strip.setPixelColor(36, c);
    break;
  
    case 5:                                                    
      strip.setPixelColor(36, c);
      strip.setPixelColor(37, c);
    break;
  
    case 6:                                                    
      strip.setPixelColor(37, c);
      strip.setPixelColor(38, c);
      strip.setPixelColor(39, c);
    break;
  
    case 7:                                                    
      strip.setPixelColor(39, c);
      strip.setPixelColor(40, c);
    break;
  
    case 8:                                                    
      strip.setPixelColor(40, c);
      strip.setPixelColor(41, c);
      strip.setPixelColor(42, c);
    break;
  
    case 9:                                                    
      strip.setPixelColor(42, c);
      strip.setPixelColor(43, c);
      strip.setPixelColor(44, c);
    break;
  
    case 10:                                                    
      strip.setPixelColor(44, c);
      strip.setPixelColor(45, c);
      strip.setPixelColor(46, c);
    break;
  
    case 11:                                                    
      strip.setPixelColor(46, c);
      strip.setPixelColor(47, c);
      strip.setPixelColor(48, c);
    break;
  
    case 12:                                                    
      strip.setPixelColor(48, c);
      strip.setPixelColor(49, c);
    break;
  
    case 13:                                                    
      strip.setPixelColor(49, c);
      strip.setPixelColor(50, c);
      strip.setPixelColor(51, c);
    break;
  
    case 14:                                                    
      strip.setPixelColor(51, c);
      strip.setPixelColor(52, c);
    break;
  
    case 15:                                                    
      strip.setPixelColor(52, c);
      strip.setPixelColor(53, c);
      strip.setPixelColor(54, c);
    break;
  
    case 16:                                                    
      strip.setPixelColor(54, c);
      strip.setPixelColor(55, c);
    break;
  
    case 17:                                                    
      strip.setPixelColor(55, c);
      strip.setPixelColor(56, c);
      strip.setPixelColor(57, c);
    break;
  
    case 18:                                                    
      strip.setPixelColor(57, c);
      strip.setPixelColor(58, c);
      strip.setPixelColor(59, c);
    break;
  
    case 19:                                                    
      strip.setPixelColor(59, c);
      strip.setPixelColor(60, c);
      strip.setPixelColor(61, c);
    break;
  
    case 20:                                                    
      strip.setPixelColor(61, c);
      strip.setPixelColor(62, c);
      strip.setPixelColor(63, c);
    break;
  
    case 21:                                                    
      strip.setPixelColor(63, c);
      strip.setPixelColor(64, c);
    break;
  
    case 22:                                                    
      strip.setPixelColor(64, c);
      strip.setPixelColor(65, c);
      strip.setPixelColor(66, c);
    break;
  
    case 23:                                                    
      strip.setPixelColor(66, c);
      strip.setPixelColor(67, c);
    break;
  
    case 24:                                                    
      strip.setPixelColor(67, c);
      strip.setPixelColor(68, c);
      strip.setPixelColor(69, c);
    break;
  
    case 25:                                                    
      strip.setPixelColor(69, c);
      strip.setPixelColor(70, c);
    break;
  
    case 26:                                                    
      strip.setPixelColor(70, c);
      strip.setPixelColor(71, c);
      strip.setPixelColor(72, c);
    break;
  
    case 27:                                                    
      strip.setPixelColor(72, c);
      strip.setPixelColor(73, c);
    break;  
}
 
    strip.show();
}

//******************************************************************************************************************
//for a 3x3 storage bin system, the number of colums = 15 (or 5 x 3) since each storage bin has 5 horizontal drawers
//the lettered columns are A through P, omit letter O because it looks like a zero. This functions takes the column 
//char and converts it to a band of LEDS that approx the column location
void set_column_led(uint32_t c, char column)
{
  
for (int i=0; i < 30; i++) 
  {
    strip.setPixelColor(i, 0);           // turn all pixels off
  }
  
 switch (column)                           //interpret first byte
  {
    //stop both stepper motors 
    case 'P':                                                    
      strip.setPixelColor(0, c);
      strip.setPixelColor(1, c);
      strip.setPixelColor(2, c); 
    break;
 
    case 'N':                                                    
      strip.setPixelColor(2, c);
      strip.setPixelColor(3, c);
      strip.setPixelColor(4, c); 
    break;
 
    case 'M':                                                    
      strip.setPixelColor(4, c);
      strip.setPixelColor(5, c);
      strip.setPixelColor(6, c); 
    break;
 
    case 'L':                                                    
      strip.setPixelColor(6, c);
      strip.setPixelColor(7, c);
    break;
   
    case 'K':                                                    
      strip.setPixelColor(8, c);
      strip.setPixelColor(9, c);
    break;
   
    case 'J':                                                    
      strip.setPixelColor(10, c);
      strip.setPixelColor(11, c);
      strip.setPixelColor(12, c); 
    break;
   
    case 'I':                                                    
      strip.setPixelColor(12, c);
      strip.setPixelColor(13, c);
      strip.setPixelColor(14, c); 
    break;
 
    case 'H':                                                    
      strip.setPixelColor(14, c);
      strip.setPixelColor(15, c);
    break;
    
    case 'G':                                                    
      strip.setPixelColor(16, c);
      strip.setPixelColor(17, c);
    break;
    
    case 'F':                                                    
      strip.setPixelColor(18, c);
      strip.setPixelColor(19, c);
    break;
    
    case 'E':                                                    
      strip.setPixelColor(20, c);
      strip.setPixelColor(21, c);
      strip.setPixelColor(22, c); 
    break;
    
    case 'D':                                                    
      strip.setPixelColor(22, c);
      strip.setPixelColor(23, c);
    break;
    
    case 'C':                                                    
      strip.setPixelColor(24, c);
      strip.setPixelColor(25, c); 
    break;
    
    case 'B':                                                    
      strip.setPixelColor(26, c);
      strip.setPixelColor(27, c);
    break;
    
    case 'A':                                                    
      strip.setPixelColor(28, c);
      strip.setPixelColor(29, c);
    break;  
  }
    strip.show();
}

//*****************************************************************************************************
//move z motor to home switch position
void home_z_axis()
{
  speed_zSet = 30;                    //set stepper to slow speed when homing
  
  digitalWrite(4, LOW);              //set z stepper direction pin low to move z axis up
  cli();                             //atomic operation
  steps_z = 65000;                   //do lots of steps
  sei();     
  
  while(digitalRead(2)){}           //wait for z switch to be pressed

  cli();                             //atomic operation
  steps_z = 0 ;                      //turn off steppers
  sei();    
 
  steps_z_absolute = 0;              //zero z axis absolute position
  
  Serial.print("1");                  //send one to indicate homing done  
   
}

//*****************************************************************************************************
//move x motor to home switch position
void home_x_axis()
{
 speed_xSet = 30;                    //set stepper to slow speed when homing
  
  digitalWrite(6, HIGH);              //set x stepper direction pin high to move x axis left
  cli();                             //atomic operation
  steps_x = 65000;                   //do lots of steps
  sei();     
  
  while(digitalRead(3)) {}              //wait for x switch to be pressed

  cli();                             //atomic operation
  steps_x = 0 ;                      //turn off steppers
  sei();    
  
  steps_x_absolute = 0;              //zero x axis absolute position
  
   Serial.print("1");                //send one to indicate homing done
 
}

//*****************************************************************************************************
//transmit state of z home switch
void read_z_switch()
{
  if (digitalRead(2))               //if pin high switch is off 
  {
  Serial.print("0");  
  }
  else                              //pin is low, switch is on
  {
  Serial.print("1");  
  }
}

//*****************************************************************************************************
//transmit state of x home switch
void read_x_switch()
{
  if (digitalRead(3))               //if pin high switch is off 
  {
  Serial.print("0");  
  }
  else                              //pin is low, switch is on
  {
  Serial.print("1"); 
  }  
}

//**********************************************************************************************************
//This is modified code from link provided by Adafruit Industries
// "Larson scanner" = Cylon/KITT bouncing light effect
//hard coded for 30 leds along the top of storagebot
void scanner(uint8_t r, uint8_t g, uint8_t b, uint8_t wait) {
  int i, j, pos, dir;

  pos = 0;
  dir = 1;

// for(i=0; i<((strip.numPixels()-1) * 2); i++) {
   for(i=0; i<((30-1) * 2); i++) {

    //  for(i=0; i<((strip.numPixels()-1)); i++) {
    // Draw 9 pixels centered on pos.  setPixelColor() will clip
    // any pixels off the ends of the strip, no worries there.
    // we'll make the colors dimmer at the edges for a nice pulse
    // look
    strip.setPixelColor(pos - 4, strip.Color(r/16, g/16, b/16));
    strip.setPixelColor(pos - 3, strip.Color(r/8, g/8, b/8));
    strip.setPixelColor(pos - 2, strip.Color(r/4, g/4, b/4));
    strip.setPixelColor(pos - 1, strip.Color(r/2, g/2, b/2));
    strip.setPixelColor(pos, strip.Color(r, g, b));
    strip.setPixelColor(pos + 1, strip.Color(r/2, g/2, b/2));
    strip.setPixelColor(pos + 2, strip.Color(r/4, g/4, b/4));
    strip.setPixelColor(pos + 3, strip.Color(r/8, g/8, b/8));
    strip.setPixelColor(pos + 4, strip.Color(r/16, g/16, b/16));

    strip.show();
    delay(wait);
    // If we wanted to be sneaky we could erase just the tail end
    // pixel, but it's much easier just to erase the whole thing
    // and draw a new one next time.
    for(j=-4; j<= 4; j++) 
  //      strip.setPixelColor(pos+j, strip.Color(0,0,0));
     strip.setPixelColor(pos+j, strip.Color(0,0,5));   
    // Bounce off ends of strip
    pos += dir;
    if(pos < 0) {
      pos = 1;
      dir = -dir;
//    } else if(pos >= strip.numPixels()) {
      } else if(pos >= 30) {
//      pos = strip.numPixels() - 2;
      pos = 30 - 2;

      dir = -dir;
    }
  }
}

